body {
    font-family: sans-serif;
    font-size: 0.9em;
}
#lhc {
    position: absolute;
    top: 0;
    left: 0;
    width: 280px;
    height: 100%;
    background-color: silver;
    color: black;
    border-right: solid black 0px;
}
#logtree {
    margin: 0.4em;
    height: 90%;
    padding: 0.2em;
    font-size: smaller;
}
#selected-entry {
    margin-left: 310px;
}
.tree-group, .tree-item-entry {
    margin-bottom: 0.6em;
}
.tree-item {
    margin-left: 0.8em;
}
.tree-group > div > a {
    font-weight:bold;
}
.tree-group a, .tree-item-entry a {
    text-decoration: none;
    color: black;
    background-color:silver;
    padding-top: 0.2em;
    padding-bottom: 0.2em;
}
.tree-group a:hover, .tree-item-entry a:hover {
    color:black;
    border:dashed gray 1px;
}

.entry-value {
  margin-left: 0.6em;
  font-size: 12px;
  font-family:monospace;
}